@@ -109,7 +109,8 @@
   # str(obj,grid=FALSE)
    mUnits <- .getPrm(arglist,name="unit(s)*",class=c("expression","character")
                     ,default=NA_character_)
-   if ((!is.na(mUnits)[1])&&(length(mUnits)==length(units)))
+   if (((is.expression(mUnits))||((is.character(mUnits))&&(is.na(mUnits[1]))))&&
+         (length(mUnits)==length(units)))
       units <- mUnits
    else if ((is.null(units))||(all(!nchar(units)))) {
       units <- .getPrm(arglist,name="unit(s)*",class=c("expression","character")
