@@ -25,7 +25,7 @@
             op <- options(warn=0)
             requireNamespace("rgdal",quietly=.isPackageInUse())
             a <- .shp.read(obj)
-           # a <- spTransform(a,session_grid()$proj4)
+           # a <- spTransform(a,session_grid()$crs)
            # ret <- .panel_plot(a,add=TRUE,...)
             ret <- sp::plot(a,add=TRUE,...)
             options(op)
@@ -41,7 +41,7 @@
          ret <- panel_raster(read_gdal(obj),...)
       }
       else if ((getOption("ursaPngScale")==1)&&
-             (.lgrep("\\+proj=merc",session_proj4())>0)&&
+             (.lgrep("\\+proj=merc",session_crs())>0)&&
              ((TRUE)||(.lgrep(obj,.tileService())))) {
          ret <- panel_raster(obj,...)
       }
@@ -52,15 +52,21 @@
       ct <- panel_raster(obj,...)
    }
    else if (is_spatial(obj)) {
-      oprj <- spatial_proj4(obj)
-      sprj <- session_proj4()
-      if (nchar(sprj)>2) {
-         oprj2 <- .gsub("\\+wktext\\s","",oprj)
-         sprj2 <- .gsub("\\+wktext\\s","",sprj)
-         oprj2 <- .gsub("(^\\s|\\s$)","",oprj2)
-         sprj2 <- .gsub("(^\\s|\\s$)","",sprj2)
-         if (!identical(oprj2,sprj2)) {
-            obj <- spatial_transform(obj,sprj)
+      oprj <- spatial_crs(obj)
+      sprj <- session_crs()
+      if (!identical(oprj,sprj)) {
+         if ((is.list(oprj))&&("input" %in% names(oprj)))
+            oprj <- oprj[["input"]]
+         if ((is.list(sprj))&&("input" %in% names(sprj)))
+            sprj <- sprj[["input"]]
+         if (nchar(sprj)>2) {
+            oprj2 <- .gsub("\\+wktext\\s","",oprj)
+            sprj2 <- .gsub("\\+wktext\\s","",sprj)
+            oprj2 <- .gsub("(^\\s|\\s$)","",oprj2)
+            sprj2 <- .gsub("(^\\s|\\s$)","",sprj2)
+            if (!identical(oprj2,sprj2)) {
+               obj <- spatial_transform(obj,sprj)
+            }
          }
       }
       geoType <- spatial_geotype(obj)
@@ -162,6 +168,11 @@
                   }
                }
             }
+            alpha <- .getPrm(arglist2,"alpha",default=1)
+            if (alpha<1) {
+               val <- t(grDevices::col2rgb(col,alpha=TRUE))
+               col <- grDevices::rgb(val[,1],val[,2],val[,3],val[,4]*alpha,max=255)
+            }
             if (!is.null(col))
                arglist2$col <- unclass(col)
          }
@@ -172,7 +183,9 @@
             arglist2$col <- ct$colortable[ct$index]
          }
          else
-            arglist2$col <- ifelse(is_spatial_lines(obj),"black","transparent")
+            arglist2$col <- ifelse(is_spatial_lines(obj),"black"
+                                  ,ifelse(T,ursa_colortable(colorize("",alpha=0.2))
+                                           ,"transparent"))
          if (.lgrep("point",geoType)) {
             if (!.isPackageInUse())
                message("'ursa'-dev: check 'https://github.com/exaexa/scattermore' for points plot")
@@ -192,6 +205,8 @@
             if (!.lgrep("lwd",names(arglist2))) {
                arglist2$lwd <- 0.5
             }
+            arglist2$col <- "black"
+            str(arglist2)
          }
          else if (.lgrep("lines",geoType)) {
             if (!.lgrep("lwd",names(arglist2))) {
@@ -351,7 +366,7 @@
    if ((TRUE)&&(.isSF(obj))&&(.lgrep("(dens|angl)",names(arglist)))) {
      # arglist$add <- NULL
       if (!.isPackageInUse())
-         message("'sf' cannot deal with fill patterns; converted to 'Spatial'")
+         message("'sf' doesn't deal with fill patterns; converted to 'Spatial'")
       if (TRUE)
          obj <- sf::as_Spatial(obj)
       else if (FALSE) {
