@@ -1,14 +1,19 @@
 # wrappers to spatial (not raster) objects
-# wrappers to spatial (not raster) objects
-
-'spatial_crs' <- 'spatial_proj4' <- function(obj,verbose=FALSE) {
+#.syn('spatial_crs',0,...)
+'spatial_proj4' <- 'spatial_proj' <- 'spatial_crs' <- function(obj,verbose=FALSE) {
+   if (!is.null(attr(obj,"crs"))) {
+      res <- attr(obj,"crs")
+      if (inherits(res,"crs"))
+         return(res$proj4string)
+      return(res)
+   }
    isUrsa <- is.ursa(obj) | is.ursa(obj,"grid")
    isPrm <- is.numeric(obj) | is.character(obj)
    isSF <- .isSF(obj)
    isSP <- .isSP(obj)
    if (verbose)
       print(data.frame(ursa=isUrsa,sf=isSF,sp=isSP,prm=isPrm,row.names="engine"))
-   if (isSF) {
+   if ((isSF)||(inherits(obj,"crs"))) {
       return(sf::st_crs(obj)$proj4string)
    }
    if (isSP) {
@@ -25,13 +30,18 @@
       return(NA_character_)
    }
    if (isUrsa)
-      return(ursa_proj4(obj))
+      return(ursa_crs(obj))
    if (isPrm) {
       return(.epsg2proj4(obj,verbose=verbose,force=TRUE))
    }
+   if (isTRUE(all(sapply(obj,.isSF))))
+      return(lapply(obj,spatial_crs,verbose=verbose))
+   if (isTRUE(all(sapply(obj,.isSP))))
+      return(lapply(obj,spatial_crs,verbose=verbose))
    return(NULL)
 }
-'spatial_crs<-' <- 'spatial_proj4<-' <- function(obj,verbose=FALSE,value) {
+#.syn('spatial_crs<-',0,...)
+'spatial_proj4<-' <- 'spatial_proj<-' <- 'spatial_crs<-' <- function(obj,verbose=FALSE,value) { 
    isSF <- .isSF(obj)
    isSP <- .isSP(obj)
    if (verbose)
@@ -59,6 +69,10 @@
    if (isSP) {
       return(sp::geometry(obj))
    }
+   if (isTRUE(all(sapply(obj,.isSF))))
+      return(lapply(obj,spatial_geometry,verbose=verbose))
+   if (isTRUE(all(sapply(obj,.isSP))))
+      return(lapply(obj,spatial_geometry,verbose=verbose))
    return(NULL)
 }
 'spatial_geometry<-' <- function(obj,verbose=FALSE,value) {
@@ -107,14 +121,21 @@
 'spatial_bbox' <- function(obj,verbose=FALSE) {
    isSF <- .isSF(obj)
    isSP <- .isSP(obj)
+   if ((!isSF)&&(!isSP)&&(is.list(obj))) {
+      if (isTRUE(all(sapply(obj,function(o) isTRUE(.isSF(o)) | isTRUE(.isSP(o)))))) {
+         return(lapply(obj,spatial_bbox,verbose=verbose))
+      }
+   }
    if (verbose)
       print(data.frame(sf=isSF,sp=isSP,row.names="engine"))
-   if (T & .lgrep("\\+proj=longlat",spatial_crs(obj))) {
+   res <- NULL
+   isLongLat <- .lgrep("\\+proj=longlat",spatial_crs(obj))>0
+   if (F & isLongLat) {
       xy <- spatial_coordinates(obj)
       if (F & isSF) {
          xy <- spatial_coordinates(sf::st_cast(spatial_geometry(obj),"POINT"))
          res <- c(xmin=min(xy[,1]),ymin=min(xy[,2]),xmax=max(xy[,1]),ymax=max(xy[,2]))
-         return(res)
+        # return(res)
       }
       if (T | isSP) {
          xy <- spatial_coordinates(obj)
@@ -123,24 +144,94 @@
          if (is.list(xy)) ## deprecated and resored 20190930
             xy <- do.call(rbind,xy)
          res <- c(xmin=min(xy[,1]),ymin=min(xy[,2]),xmax=max(xy[,1]),ymax=max(xy[,2]))
-         return(res)
+        # return(res)
       }
    }
-   if (isSF) {
+   if ((is.null(res))&&(isSF)) {
       res <- sf::st_bbox(obj)
       rname <- names(res)
       res <- as.numeric(res)
       names(res) <- rname
-      return(res)
+      attr(res,"crs") <- spatial_crs(obj)
+     # return(res)
    }
-   if (isSP) {
+   if ((is.null(res))&&(isSP)) {
       res <- c(sp::bbox(obj))
       if (length(res)==6)
          res <- res[c(1,2,4,5)]
       names(res) <- c("xmin","ymin","xmax","ymax")
-      return(res)
+      attr(res,"crs") <- spatial_crs(obj)
+     # return(res)
    }
-   NULL
+   ##~ cat("----------------\n")
+   ##~ if ((isLongLat)&&(res["xmin"]<(-180+1e-6))&&(res["xmax"]>(180-1e-6))) {
+      ##~ print("HERE")
+   ##~ }
+   ##~ print(res)
+   ##~ cat("----------------\n")
+   ##~ q()
+  # return(res)
+   if (!is.null(res)) {
+      if (!isLongLat)
+         return(res)
+      dg180 <- 0.1
+      is180 <- (res["xmin"]<=(-180+dg180))&&(res["xmax"]>=(180-dg180))
+      if (!is180)
+         return(res)
+      dg90 <- -1e-6
+      is90 <- (res["ymin"]<=(-90+dg90))||(res["ymax"]>=(90-dg90))
+      if (is90)
+         return(res)
+      if (skip_dev <- !TRUE)
+         return(res)
+     # if ((TRUE)&&(res["xmin"]>0)&&(res["xmax"]<0))
+     #    res["xmax"] <- res["xmax"]+360
+      if (T) {
+         xy <- unname(spatial_coordinates(obj))
+         if (T) {
+            for (i in seq(4)) {
+               if (isFALSE(any(sapply(xy,is.list))))
+                  break
+               if (length(ind <- which(sapply(xy,is.list)))) {
+                  xy <- c(xy[-ind],unlist(xy[ind],recursive=FALSE))
+               }
+            }
+         }
+         else {
+            repeat({
+               if (isFALSE(any(sapply(xy,is.list))))
+                  break
+               xy <- unlist(xy,recursive=FALSE)
+            })
+         }
+         if (is.list(xy))
+            xy <- do.call("rbind",xy)
+         x1 <- xy[,1]
+         x2 <- x1
+         ind <- which(x2<(0))
+         if (length(x2))
+            x2[ind] <- x2[ind]+360
+         sd1 <- sd(x1)
+         sd2 <- sd(x2)
+         if (verbose) {
+            print(summary(x1))
+            print(summary(x2))
+            print(c(sd1=sd1,sd2=sd2))
+         }
+         if (sd1<sd2)
+            return(res)
+         res[c(1,3)] <- range(x2)
+      }
+      else {
+         d2 <- spatialize(obj,style="merc",verbose=verbose) ## sp: failed for 90.0N 
+         res <- matrix(spatial_bbox(d2),ncol=2,byrow=FALSE)
+         res <- .project(res,spatial_crs(d2),inv=TRUE)
+         res <- c(t(res))
+         names(res) <- c("xmin","ymin","xmax","ymax")
+         attr(res,"crs") <- spatial_crs(obj)
+      }
+   }
+   res
 }
 'spatial_bbox<-' <- function(obj,verbose=FALSE,value) {
    isSF <- .isSF(obj)
@@ -174,6 +265,11 @@
 'spatial_fields' <- 'spatial_colnames' <- function(obj,verbose=FALSE) {
    isSF <- .isSF(obj)
    isSP <- .isSP(obj)
+   if ((!isSF)&&(!isSP)&&(is.list(obj))) {
+      if (isTRUE(all(sapply(obj,function(o) isTRUE(.isSF(o)) | isTRUE(.isSP(o)))))) {
+         return(lapply(obj,spatial_fields,verbose=verbose))
+      }
+   }
    if (verbose)
       print(data.frame(sf=isSF,sp=isSP,row.names="engine"))
    if (isSF) {
@@ -206,9 +302,15 @@
       print(data.frame(sf=isSF,sp=isSP,row.names="engine"))
    if (isSF) {
       oldvalue <- names(attr(obj,"agr"))
-      colnames(obj)[match(oldvalue,colnames(obj))] <- value
-     # colnames(obj)[-match(attr(obj,"sf_column"),colnames(obj))] <- value
-      names(attr(obj,"agr")) <- value
+      if (!is.null(oldvalue)) {
+         colnames(obj)[match(oldvalue,colnames(obj))] <- value
+        # colnames(obj)[-match(attr(obj,"sf_column"),colnames(obj))] <- value
+         names(attr(obj,"agr")) <- value
+      }
+      else {
+         ind <- grep(attr(obj,"sf_column"),names(obj),invert=TRUE,value=FALSE)
+         colnames(obj)[ind] <- value
+      }
    }
    if (isSP) {
       colnames(spatial_data(obj)) <- value 
@@ -218,14 +320,22 @@
 'spatial_data' <- function(obj,subset=".+",drop=NA,verbose=FALSE) {
    isSF <- .isSF(obj)
    isSP <- .isSP(obj)
+   if ((!isSF)&&(!isSP)&&(is.list(obj))) {
+      if (isTRUE(all(sapply(obj,function(o) isTRUE(.isSF(o)) | isTRUE(.isSP(o)))))) {
+         return(lapply(obj,spatial_data,subset=subset,drop=drop,verbose=verbose))
+      }
+   }
    if (verbose)
       print(data.frame(sf=isSF,sp=isSP,row.names="engine"))
    if (isSF) {
       if (inherits(obj,"sfc"))
          return(NULL)
      # res <- obj
-     # sf::st_geometry(res) <- NULL
-     # attributes(res) <- attributes(res)[c("names","row.names","class")]
+      ##~1 sf::st_geometry(res) <- NULL
+      ##~1 attributes(res) <- attributes(res)[c("names","row.names","class")]
+      ##~2 res[[attr(res,"sf_column")]] <- NULL
+      ##~2 attr(res,"sf_column") <- NULL
+      ##~2 class(res) <- grep("^(sf|sfc)$",class(res),value=TRUE,invert=TRUE)
       res <- sf::st_set_geometry(obj,NULL)
    }
    else if (isSP) {
@@ -273,8 +383,13 @@
 'spatial_transform' <- function(obj,crs,verbose=FALSE,...) {
    isSF <- .isSF(obj)
    isSP <- .isSP(obj)
+   if ((!isSF)&&(!isSP)&&(is.list(obj))) {
+      if (isTRUE(all(sapply(obj,function(o) isTRUE(.isSF(o)) | isTRUE(.isSP(o)))))) {
+         return(lapply(obj,spatial_transform,crs=crs,verbose=verbose))
+      }
+   }
    if (missing(crs))
-      crs <- session_proj4()
+      crs <- session_crs()
    else if ((is.ursa(crs))||(is.ursa(crs,"grid")))
       crs <- ursa(crs,"crs")
    if (verbose)
@@ -574,7 +689,7 @@
       }
       else {
          res <- sp::SpatialLinesLengths(obj
-                         ,longlat=.lgrep("\\+proj=longlat",spatial_proj4(obj))>0)
+                         ,longlat=.lgrep("\\+proj=longlat",spatial_crs(obj))>0)
       }
       return(res)
    }
@@ -815,7 +930,12 @@
       if (inherits(y,"sf"))
          sf::st_agr(y) <- "constant"
      # x_
-      res <- sf::st_intersection(x,y)
+      res <- try(sf::st_intersection(x,y))
+      if (inherits(res,"try-error")) {
+         if (length(grep("st_crs\\(x\\) == st_crs\\(y\\) is not TRUE"
+                        ,as.character(res))))
+         res <- sf::st_intersection(x,spatial_transform(y,x))
+      }
       if (FALSE) {
         # spatial_geometry(res) <- sf:::st_cast_sfc_default(spatial_geometry(res))
       }
@@ -851,9 +971,13 @@
             if (length(na.omit(match(unique(geotype)
                                     ,c("POLYGON","MULTIPOLYGON"))))==2) {
                if (length(ind2 <- .grep("collection",geotype))) {
-                  res <- spatial_bind(res[-ind2,]
-                               ,sf::st_collection_extract(res[ind2,],"POLYGON"))
-                  geotype <- as.character(spatial_geotype(res,each=TRUE))
+                  opWC <- options(warn=-1)
+                  res2 <- sf::st_collection_extract(res[ind2,],"POLYGON")
+                  options(opWC)
+                  if (spatial_count(res2)) {
+                     res <- spatial_bind(res[-ind2,],res2)
+                     geotype <- as.character(spatial_geotype(res,each=TRUE))
+                  }
                }
                res <- sf::st_cast(res[grep("POLYGON",geotype),],"MULTIPOLYGON")
             }
@@ -944,7 +1068,12 @@
          sf::st_agr(x) <- "constant"
       if (inherits(y,"sf"))
          sf::st_agr(y) <- "constant"
-      res <- sf::st_difference(x,y)
+      res <- try(sf::st_difference(x,y))
+      if (inherits(res,"try-error")) {
+         if (length(grep("st_crs\\(x\\) == st_crs\\(y\\) is not TRUE"
+                        ,as.character(res))))
+         res <- sf::st_difference(x,spatial_transform(y,x))
+      }
       return(res)
    }
    else if (isSP) {
@@ -971,7 +1100,12 @@
          sf::st_agr(x) <- "constant"
       if (inherits(y,"sf"))
          sf::st_agr(y) <- "constant"
-      res <- sf::st_sym_difference(x,y)
+      res <- try(sf::st_sym_difference(x,y))
+      if (inherits(res,"try-error")) {
+         if (length(grep("st_crs\\(x\\) == st_crs\\(y\\) is not TRUE"
+                        ,as.character(res))))
+         res <- sf::st_sym_difference(x,spatial_transform(y,x))
+      }
       return(res)
    }
    else if (isSP) {
@@ -1210,3 +1344,26 @@
    attr(obj,basename(tempfile())) <- NULL ## dummy
    obj
 }
+'spatial_grid' <- function(obj) {
+   if ((is.numeric(obj))&&(length(obj)==4)) {
+      bbox <- obj
+      if (!is.null(attr(obj,"crs")))
+         crs <- attr(obj,"crs")
+      else
+         crs <- session_crs()
+   }
+   else {
+      bbox <- spatial_bbox(obj)
+      if ((bbox["xmin"]==bbox["xmax"])||(bbox["ymin"]==bbox["ymax"]))
+         bbox <- bbox+100*c(-1,-1,1,1)
+      crs <- spatial_crs(obj)
+   }
+   if ((.lgrep("\\+proj=longlat",crs))&&(bbox["xmax"]<0)&&(bbox["xmin"]>0))
+      bbox["xmax"] <- bbox["xmax"]+360
+   nc <- (bbox["xmax"]-bbox["xmin"])
+   nr <- (bbox["ymax"]-bbox["ymin"])
+   res <- max(nc,nr)/640
+   p <- pretty(res)
+   res <- p[which.min(abs(res-p))]
+   regrid(setbound=unname(bbox),crs=crs,res=res)
+}
