@@ -85,6 +85,23 @@
    .ursaCacheWrite(was0,append=FALSE)
    return(invisible(NULL))
 }
+'.ursaCacheExpired' <- function(value) {
+   if (is.character(value)) {
+      if (!.lgrep("\\-",value))
+         value <- paste0("-",value)
+      if (!.lgrep("\\d\\s\\D",value))
+         value <- gsub("(^.*\\d)(\\D.+$)","\\1 \\2",value)
+      expired <- as.integer(tail(seq(Sys.time(),len=2,by=value),1))
+     # expire <- as.POSIXlt(tail(seq(Sys.time(),len=2,by=value),1),tz="UTC")
+      attr(expired,"cache") <- TRUE
+   }
+   else {
+      expired <- as.integer(tail(seq(Sys.time(),len=2,by="-1 month"),1)) ## not '+'
+     # expired <- as.POSIXlt(tail(seq(Sys.time(),len=2,by="-1 month"),1),tz="UTC")
+      attr(expired,"cache") <- value
+   }
+   expired
+}
 '.ursaCacheDownload' <- function(src,dst,method,quiet=FALSE,cache=TRUE
                                 ,mode="w",headers=NULL) {
    enc <- "UTF-8"
@@ -97,20 +114,45 @@
    }
    if (missing(dst))
       dst <- NULL
+   expired <- .ursaCacheExpired(cache)
+  # message("expired:")
+  # str(expired)
+  # str(as.POSIXct(expired,origin=.origin()))
+   cache <- attr(expired,"cache")
+   ##~ if (is.character(cache)) {
+      ##~ if (!.lgrep("\\-",cache))
+         ##~ cache <- paste0("-",cache)
+      ##~ if (!.lgrep("\\d\\s\\D",cache))
+         ##~ cache <- gsub("(^.*\\d)(\\D.+$)","\\1 \\2",cache)
+      ##~ expired <- as.POSIXlt(tail(seq(Sys.time(),len=2,by=cache),1),tz="UTC")
+      ##~ cache <- TRUE
+   ##~ }
+   ##~ else
+      ##~ expired <- as.POSIXlt(tail(seq(Sys.time(),len=2,by="-7 days"),1),tz="UTC")
    if (cache) {
+     # message("use cache")
       if (file.exists(inventory)) {
+        # message("check inventory")
          was <- utils::read.table(inventory,sep=",",encoding=enc)
          colnames(was) <- c("time","stamp","visits","size","src","dst")
          if (is.character(dst)) {
             stop("dst")
          }
-         ind <- match(src0,was$src)
+         ind <- tail(which(!is.na(match(was$src,src0))),1) ## match(src0,was$src)
          if ((length(ind))&&(!is.na(ind))) {
-            dst <- file.path(.ursaCacheDir(),was$dst[ind[1]])
+           # message("found in inventory")
+           # str(was$stamp[ind])
+           # str(as.POSIXct(was$stamp[ind],origin=.origin()))
+           # t2 <- as.POSIXlt(was$time[ind],format="%Y-%m-%dT%H:%M:%SZ",tz="UTC")
+            if (was$stamp[ind]>=expired) { ## t2>=expired
+              # message("not expired")
+               dst <- file.path(.ursaCacheDir(),was$dst[ind])
+            }
          }
       }
    }
    if ((is.null(dst))||(!file.exists(dst))) {
+     # message("download is required")
       if (!length(src))
          return(NULL)
       if (is.null(dst))
